<?php

declare(strict_types=1);

namespace NetInventors\NetiNextToolKit\Subscriber;

use NetInventors\NetiNextToolKit\Struct\PluginConfigStruct;
use NetInventors\NetiNextToolKit\Struct\ProductPropertyOptionsStruct;
use Shopware\Core\Content\Cms\Aggregate\CmsSlot\CmsSlotEntity;
use Shopware\Core\Content\Cms\CmsPageEntity;
use Shopware\Core\Content\Cms\Events\CmsPageLoadedEvent;
use Shopware\Core\Content\Cms\SalesChannel\Struct\ProductSliderStruct;
use Shopware\Core\Content\Product\Events\ProductListingResultEvent;
use Shopware\Core\Content\Product\ProductCollection;
use Shopware\Core\Content\Product\ProductEntity;
use Shopware\Core\Content\Product\SalesChannel\SalesChannelProductEntity;
use Shopware\Core\Content\Property\Aggregate\PropertyGroupOption\PropertyGroupOptionCollection;
use Shopware\Core\Content\Property\Aggregate\PropertyGroupOption\PropertyGroupOptionEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\EntitySearchResult;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsAnyFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\MultiFilter;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ProductListingSubscriber implements EventSubscriberInterface
{
    public function __construct(
        private readonly EntityRepository $productRepository,
        private readonly PluginConfigStruct $pluginConfig,
    ) {
    }

    public static function getSubscribedEvents(): array
    {
        return [
            ProductListingResultEvent::class => 'onProductListingResult',
            CmsPageLoadedEvent::class        => 'onCmsPageLoaded',
        ];
    }

    public function onCmsPageLoaded(CmsPageLoadedEvent $event): void
    {
        if (!$this->pluginConfig->isActive() || [] === $this->pluginConfig->getListingProperties()) {
            return;
        }

        $productIds       = [];
        $pageProducts     = [];
        $configProperties = $this->pluginConfig->getListingProperties();
        $context          = $event->getContext();

        /** @var CmsPageEntity $page */
        foreach ($event->getResult()->getElements() as $page) {
            $slots = $page->getElementsOfType('product-slider');

            /** @var CmsSlotEntity $slot */
            foreach ($slots as $slot) {
                $slider = $slot->getData();
                if (!$slider instanceof ProductSliderStruct) {
                    continue;
                }

                $products = $slider->getProducts();
                if (!$products instanceof ProductCollection) {
                    continue;
                }

                foreach ($products->getElements() as $product) {
                    if (null === $product->getParentId() && 1 > $product->getChildCount()) {
                        continue;
                    }

                    $options = $product->getOptions();
                    if (!$options instanceof PropertyGroupOptionCollection) {
                        continue;
                    }

                    $propertyGroupIds = $options->getPropertyGroupIds();
                    $wantedProperties = array_intersect($configProperties, $propertyGroupIds);

                    if ([] === $wantedProperties) {
                        continue;
                    }

                    $pageProducts[] = $product;
                    $productIds[]   = $product->getParentId() ?? $product->getId();
                }
            }
        }

        if ([] === $productIds) {
            return;
        }

        $propertyProducts    = $this->getProductsByParentOrId($productIds, $context);
        $propertiesForSlider = $this->sortPropertyProducts($propertyProducts, $configProperties);
        $this->assignProductPropertyExtensions($pageProducts, $propertiesForSlider);
    }

    public function onProductListingResult(ProductListingResultEvent $event): void
    {
        if (!$this->pluginConfig->isActive() || [] === $this->pluginConfig->getListingProperties()) {
            return;
        }

        $products = $event->getResult()->getElements();
        $context  = $event->getContext();
        /** @var string[] $productIds */
        $productIds       = [];
        $configProperties = $this->pluginConfig->getListingProperties();

        /** @var SalesChannelProductEntity $product */
        foreach ($products as $product) {
            if (null === $product->getParentId() && 1 > $product->getChildCount()) {
                continue;
            }

            $options = $product->getOptions();
            if (!$options instanceof PropertyGroupOptionCollection) {
                continue;
            }

            $propertyGroupIds = $options->getPropertyGroupIds();
            $wantedProperties = array_intersect($configProperties, $propertyGroupIds);

            if ([] === $wantedProperties) {
                continue;
            }

            $productIds[] = $product->getParentId() ?? $product->getId();
        }

        if ([] === $productIds) {
            return;
        }

        $propertyProducts     = $this->getProductsByParentOrId($productIds, $context);
        $propertiesForListing = $this->sortPropertyProducts($propertyProducts, $configProperties);
        $this->assignProductPropertyExtensions($products, $propertiesForListing);
    }

    /**
     * @param string[] $productIds
     */
    private function getProductsByParentOrId(array $productIds, Context $context): EntitySearchResult
    {
        $criteria = new Criteria();
        $criteria->addFilter(
            new MultiFilter(
                MultiFilter::CONNECTION_OR,
                [
                    new EqualsAnyFilter('id', $productIds),
                    new EqualsAnyFilter('parentId', $productIds),
                ],
            ),
        );

        $criteria->addAssociation('options.media');

        return $this->productRepository->search($criteria, $context);
    }

    /**
     * @return array<string, array<string, array<string, PropertyGroupOptionEntity>>> $properties
    */
    private function sortPropertyProducts(EntitySearchResult $propertyProducts, array $configProperties): array
    {
        $sortedProperties = [];

        /** @var ProductEntity $propertyProduct */
        foreach ($propertyProducts as $propertyProduct) {
            $options     = $propertyProduct->getOptions();
            $referenceId = $propertyProduct->getParentId() ?? $propertyProduct->getId();

            if (!$options instanceof PropertyGroupOptionCollection) {
                continue;
            }

            /** @var PropertyGroupOptionEntity $option */
            foreach ($options->getElements() as $option) {
                $groupId = $option->getGroupId();
                if (!in_array($groupId, $configProperties, true)) {
                    continue;
                }

                $sortedProperties[$referenceId][$groupId][$option->getId()] = $option;
            }
        }

        return $sortedProperties;
    }

    /**
     * @param array<string, array<string, array<string, PropertyGroupOptionEntity>>> $properties
     */
    private function assignProductPropertyExtensions(array $products, array $properties): void
    {
        /** @var SalesChannelProductEntity $product */
        foreach ($products as $product) {
            $referenceId = $product->getParentId() ?? $product->getId();

            if (array_key_exists($referenceId, $properties)) {
                $struct = new ProductPropertyOptionsStruct($properties[$referenceId]);
                $product->addExtension('netiToolKitListingProperties', $struct);
            }
        }
    }
}
