<?php

declare(strict_types=1);

namespace NetInventors\NetiNextToolKit\Subscriber;

use NetInventors\NetiNextToolKit\Struct\PluginConfigStruct;
use NetInventors\NetiNextToolKit\Struct\ProductReviewStruct;
use Shopware\Core\Content\Product\Aggregate\ProductReview\ProductReviewEntity;
use Shopware\Core\Content\Product\Events\ProductListingResultEvent;
use Shopware\Core\Content\Product\ProductEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Aggregation\Bucket\TermsAggregation;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Aggregation\Metric\AvgAggregation;
use Shopware\Core\Framework\DataAbstractionLayer\Search\AggregationResult\Bucket\TermsResult;
use Shopware\Core\Framework\DataAbstractionLayer\Search\AggregationResult\Metric\AvgResult;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsAnyFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Sorting\FieldSorting;
use Shopware\Core\Framework\Struct\ArrayStruct;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ProductReviewSubscriber implements EventSubscriberInterface
{
    public function __construct(
        private readonly EntityRepository $productReviewRepository,
        private readonly PluginConfigStruct $pluginConfig,
    ) {
    }

    public static function getSubscribedEvents(): array
    {
        return [
            ProductListingResultEvent::class => 'onProductListingResult',
        ];
    }

    public function onProductListingResult(ProductListingResultEvent $event): void
    {
        if (!$this->pluginConfig->isActive()) {
            return;
        }

        $reviewDisplayModes = $this->pluginConfig->getReviewDisplayModes();
        $showReviewAverage  = $this->pluginConfig->isShowReviewAverage();

        if (PluginConfigStruct::REVIEW_DISPLAY_NONE === $reviewDisplayModes) {
            return;
        }

        $products = $event->getResult()->getElements();
        $context  = $event->getContext();

        $productIds = [];
        foreach ($products as $product) {
            if ($product instanceof ProductEntity) {
                $productIds[] = $product->getId();
            }
        }

        if ([] === $productIds) {
            return;
        }

        $reviewCount = (PluginConfigStruct::REVIEW_DISPLAY_COUNT === $reviewDisplayModes)
            ? $this->pluginConfig->getReviewCount()
            : null;

        $groupedReviews = $this->getReviewsForProducts($productIds, $context, $reviewCount);

        $averages = [];
        if ($showReviewAverage) {
            $averages = $this->getReviewAveragesForProducts($productIds, $context);
        }

        foreach ($products as $product) {
            if (!$product instanceof ProductEntity) {
                continue;
            }

            $productId = $product->getId();
            if ('' === $productId) {
                continue;
            }

            $reviews = isset($groupedReviews[$productId]) && is_array($groupedReviews[$productId])
                ? $groupedReviews[$productId]
                : [];

            $product->addExtension('productReviews', new ProductReviewStruct($reviews));

            if ($showReviewAverage) {
                $average = isset($averages[$productId]) ? (float) $averages[$productId] : 0.0;
                $product->addExtension('reviewAverage', new ArrayStruct(['value' => $average]));
            }
        }
    }

    /**
     * @param string[] $productIds
     */
    private function getReviewsForProducts(array $productIds, Context $context, ?int $reviewCount): array
    {
        $reviewDisplayModes = $this->pluginConfig->getReviewDisplayModes();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsAnyFilter('productId', $productIds));
        $criteria->addSorting(new FieldSorting('createdAt', FieldSorting::DESCENDING));
        $criteria->addAssociation('customer');
        $criteria->addAssociation('product');

        if (null !== $reviewCount && PluginConfigStruct::REVIEW_DISPLAY_ALL === $reviewDisplayModes) {
            $criteria->setLimit($reviewCount);
        }

        $reviews = $this->productReviewRepository->search($criteria, $context)->getEntities();

        $grouped = [];
        /** @var string $productId */
        foreach ($reviews as $review) {
            if (!$review instanceof ProductReviewEntity) {
                continue;
            }

            $productId = $review->getProductId();

            if ('' === $productId) {
                continue;
            }

            if (!isset($grouped[$productId])) {
                $grouped[$productId] = [];
            }

            if (null === $reviewCount || count($grouped[$productId]) < $reviewCount) {
                $grouped[$productId][] = $review;
            }
        }

        return $grouped;
    }

    /**
     * @param string[] $productIds
     * @psalm-suppress InternalMethod
     *
     */
    private function getReviewAveragesForProducts(array $productIds, Context $context): array
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsAnyFilter('productId', $productIds));
        $criteria->addAggregation(
            new TermsAggregation(
                'products_avg',
                'productId',
                null,
                null,
                new AvgAggregation('avg_rating', 'points'),
            ),
        );
        $result           = $this->productReviewRepository->aggregate($criteria, $context);
        $termsAggregation = $result->get('products_avg');

        $averages = [];
        if ($termsAggregation instanceof TermsResult) {
            foreach ($termsAggregation->getBuckets() as $bucket) {
                $productId = $bucket->getKey();
                if (null === $productId) {
                    continue;
                }
                $avgResult = $bucket->getResult();

                $averages[$productId] = ($avgResult instanceof AvgResult) ? $avgResult->getAvg() : 0.0;
            }
        }

        return $averages;
    }
}
